@@ -20,7 +20,7 @@
 /*                                                                        */
 /**************************************************************************/
 
-/* $Id: wrappers.c 1347 2007-06-20 07:40:34Z guesdon $ */
+/* $Id$ */
 
 #include <string.h>
 #include <caml/mlvalues.h>
@@ -50,6 +50,36 @@ value alloc_memblock_indirected (asize_t size)
     return ret;
 }
 
+static long gc_speed = 10;
+
+CAMLprim value ml_set_gc_speed(value percent)
+{
+  gc_speed = Int_val(percent);
+  return Val_unit;
+}
+
+CAMLprim value ml_get_gc_speed(value unit)
+{
+  return Val_int(gc_speed);
+}
+
+/* Reimplement the old behaviour of alloc_custom, to be sure that
+   values do not move around */
+CAMLexport value ml_alloc_custom(struct custom_operations * ops,
+                                 uintnat size,
+                                 mlsize_t mem,
+                                 mlsize_t max)
+{
+  mlsize_t wosize;
+  value result;
+
+  wosize = 1 + (size + sizeof(value) - 1) / sizeof(value);  
+  result = caml_alloc_shr(wosize, Custom_tag);
+  Custom_ops_val(result) = ops;
+  caml_adjust_gc_speed(mem*gc_speed, max*100);
+  return caml_check_urgent_gc(result);
+}
+
 CAMLprim value ml_some (value v)
 {
      CAMLparam1(v);
@@ -61,8 +91,7 @@ CAMLprim value ml_some (value v)
 value ml_cons (value v, value l)
 {
   CAMLparam2(v, l);
-  CAMLlocal1(cell);
-  cell = alloc_small(2, Tag_cons);
+  value cell = alloc_small(2, Tag_cons);
   Field(cell, 0) = v;
   Field(cell, 1) = l;
   CAMLreturn(cell);
@@ -70,7 +99,7 @@ value ml_cons (value v, value l)
 
 void ml_raise_null_pointer ()
 {
-  static value * exn = NULL;
+  static const value * exn = NULL;
   if (exn == NULL)
       exn = caml_named_value ("null_pointer");
   raise_constant (*exn);
@@ -95,17 +124,20 @@ value copy_string_or_null (const char*str)
     return copy_string (str ? (char*) str : "");
 }
 
+value Val_option_string (const char *s)
+{ return Val_option (s, Val_string); }
+
 CAMLprim value *ml_global_root_new (value v)
 {
     value *p = stat_alloc(sizeof(value));
     *p = v;
-    register_global_root (p);
+    caml_register_global_root (p);
     return p;
 }
 
 CAMLexport void ml_global_root_destroy (void *data)
 {
-    remove_global_root ((value *)data);
+    caml_remove_global_root ((value *)data);
     stat_free (data);
 }
 
@@ -145,8 +177,62 @@ CAMLexport value ml_lookup_flags_getter (const lookup_info table[], int data)
   CAMLreturn(l);
 }
 
-ML_2 (ml_lookup_from_c, (lookup_info*), Int_val, 0+)
-ML_2 (ml_lookup_to_c, (lookup_info*), 0+, Val_int)
+ML_2 (ml_lookup_from_c, Lookup_info_val, Int_val, 0+)
+ML_2 (ml_lookup_to_c, Lookup_info_val, 0+, Val_int)
+
+gchar **
+strv_of_string_list (value list)
+{
+  gchar **str_v;
+  gsize i, len;
+  value l;
+  for (len = 0, l = list; l != Val_emptylist; len++, l = Field (l, 1))
+    ;
+  l = list;
+  str_v = g_new (gchar *, len+1);
+  for (i = 0; i < len; i++)
+    {
+      str_v[i] = g_strdup (String_val (Field (l, 0)));
+      l = Field (l, 1);
+    }
+  str_v[len] = NULL;
+  return str_v;
+}
+
+value
+string_list_of_strv (const gchar * const *v)
+{
+  CAMLparam0();
+  CAMLlocal4(head, l, cell, s);
+  gsize i;
+  if (v == NULL)
+    CAMLreturn (Val_emptylist);
+  i = 0;
+  head = l = Val_emptylist;
+  while (v[i] != NULL)
+    {
+      s = copy_string (v[i]);
+      cell = alloc_small (2, Tag_cons);
+      Field (cell, 0) = s;
+      Field (cell, 1) = Val_emptylist;
+      if (l == Val_emptylist)
+	  head = l = cell;
+      else
+	{
+	  Field (l, 1) = cell;
+	  l = cell;
+	}
+      i++;
+    }
+  CAMLreturn (head);
+}
+
+value
+string_list_of_strv2 (gchar **v) {
+  return string_list_of_strv((const gchar* const*)v);
+}
+
+
 
 #ifdef ABSVALUE
 CAMLexport intnat Long_val(value x)  { return (intnat)x >> 1; }
